# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .card_payment_method import CardPaymentMethod
from .cash_payment_method import CashPaymentMethod
from .check_payment_method import CheckPaymentMethod
from .patient_payment import PatientPayment
from .patient_payment_id import PatientPaymentId
from .patient_payment_sort_field import PatientPaymentSortField
from .patient_payments_page import PatientPaymentsPage
from .payment_method import PaymentMethod, PaymentMethod_Card, PaymentMethod_Cash, PaymentMethod_Check
from .payment_method_detail import PaymentMethodDetail

__all__ = [
    "CardPaymentMethod",
    "CashPaymentMethod",
    "CheckPaymentMethod",
    "PatientPayment",
    "PatientPaymentId",
    "PatientPaymentSortField",
    "PatientPaymentsPage",
    "PaymentMethod",
    "PaymentMethodDetail",
    "PaymentMethod_Card",
    "PaymentMethod_Cash",
    "PaymentMethod_Check",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ReportTransmissionCode(str, enum.Enum):
    CBM = "BM"
    """
    By Mail
    """

    CEL = "EL"
    """
    Electronically Only
    """

    CFX = "FX"
    """
    By Fax
    """

    _UNKNOWN = "__REPORTTRANSMISSIONCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ReportTransmissionCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        cbm: typing.Callable[[], T_Result],
        cel: typing.Callable[[], T_Result],
        cfx: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ReportTransmissionCode.CBM:
            return cbm()
        if self is ReportTransmissionCode.CEL:
            return cel()
        if self is ReportTransmissionCode.CFX:
            return cfx()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.claim_id import ClaimId
from ....commons.types.invoice_id import InvoiceId
from ....commons.types.page_token import PageToken
from ....commons.types.patient_external_id import PatientExternalId
from ....commons.types.provider_id import ProviderId
from ....commons.types.service_line_id import ServiceLineId
from ....commons.types.sort_direction import SortDirection
from ....financials.types.allocation_create import AllocationCreate
from ....financials.types.invoice_update import InvoiceUpdate
from ....financials.types.note_update import NoteUpdate
from ....financials.types.patient_transaction_source import PatientTransactionSource
from ....financials.types.refund_reason import RefundReason
from ....financials.types.refund_reason_update import RefundReasonUpdate
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.patient_refund import PatientRefund
from .types.patient_refund_id import PatientRefundId
from .types.patient_refund_sort_field import PatientRefundSortField
from .types.patient_refunds_page import PatientRefundsPage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        unattributed: typing.Optional[bool] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sources: typing.Optional[
            typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]
        ] = None,
        sort: typing.Optional[PatientRefundSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientRefundsPage:
        """
        Returns all patient refunds satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        patient_external_id : typing.Optional[PatientExternalId]

        claim_id : typing.Optional[ClaimId]

        service_line_id : typing.Optional[ServiceLineId]

        billing_provider_id : typing.Optional[ProviderId]

        unattributed : typing.Optional[bool]
            returns payments with unattributed allocations if set to true

        invoice_id : typing.Optional[InvoiceId]

        sources : typing.Optional[typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]]

        sort : typing.Optional[PatientRefundSortField]
            Defaults to refund_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefundsPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_refunds.v_1.get_multi()
        """
        _response = self._raw_client.get_multi(
            limit=limit,
            patient_external_id=patient_external_id,
            claim_id=claim_id,
            service_line_id=service_line_id,
            billing_provider_id=billing_provider_id,
            unattributed=unattributed,
            invoice_id=invoice_id,
            sources=sources,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PatientRefund:
        """
        Retrieves a previously created patient refund by its `patient_refund_id`.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefund

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_refunds.v_1.get(
            patient_refund_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(patient_refund_id, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        amount_cents: int,
        patient_external_id: PatientExternalId,
        allocations: typing.Sequence[AllocationCreate],
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[str] = OMIT,
        invoice: typing.Optional[InvoiceId] = OMIT,
        refund_reason: typing.Optional[RefundReason] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientRefund:
        """
        Creates a new patient refund record and returns the newly created PatientRefund object.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters
        ----------
        amount_cents : int

        patient_external_id : PatientExternalId

        allocations : typing.Sequence[AllocationCreate]

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[str]

        invoice : typing.Optional[InvoiceId]

        refund_reason : typing.Optional[RefundReason]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefund

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.financials import (
            AllocationCreate,
            AllocationTargetCreate_ServiceLineById,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_refunds.v_1.create(
            amount_cents=1,
            patient_external_id="patient_external_id",
            allocations=[
                AllocationCreate(
                    amount_cents=1,
                    target=AllocationTargetCreate_ServiceLineById(
                        value=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        )
                    ),
                ),
                AllocationCreate(
                    amount_cents=1,
                    target=AllocationTargetCreate_ServiceLineById(
                        value=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        )
                    ),
                ),
            ],
        )
        """
        _response = self._raw_client.create(
            amount_cents=amount_cents,
            patient_external_id=patient_external_id,
            allocations=allocations,
            refund_timestamp=refund_timestamp,
            refund_note=refund_note,
            invoice=invoice,
            refund_reason=refund_reason,
            request_options=request_options,
        )
        return _response.data

    def update(
        self,
        patient_refund_id: PatientRefundId,
        *,
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[NoteUpdate] = OMIT,
        invoice: typing.Optional[InvoiceUpdate] = OMIT,
        refund_reason: typing.Optional[RefundReasonUpdate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientRefund:
        """
        Updates the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[NoteUpdate]

        invoice : typing.Optional[InvoiceUpdate]

        refund_reason : typing.Optional[RefundReasonUpdate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefund

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_refunds.v_1.update(
            patient_refund_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.update(
            patient_refund_id,
            refund_timestamp=refund_timestamp,
            refund_note=refund_note,
            invoice=invoice,
            refund_reason=refund_reason,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.patient_refunds.v_1.delete(
            patient_refund_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.delete(patient_refund_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        patient_external_id: typing.Optional[PatientExternalId] = None,
        claim_id: typing.Optional[ClaimId] = None,
        service_line_id: typing.Optional[ServiceLineId] = None,
        billing_provider_id: typing.Optional[ProviderId] = None,
        unattributed: typing.Optional[bool] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sources: typing.Optional[
            typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]
        ] = None,
        sort: typing.Optional[PatientRefundSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientRefundsPage:
        """
        Returns all patient refunds satisfying the search criteria AND whose organization_id matches
        the current organization_id of the authenticated user.

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        patient_external_id : typing.Optional[PatientExternalId]

        claim_id : typing.Optional[ClaimId]

        service_line_id : typing.Optional[ServiceLineId]

        billing_provider_id : typing.Optional[ProviderId]

        unattributed : typing.Optional[bool]
            returns payments with unattributed allocations if set to true

        invoice_id : typing.Optional[InvoiceId]

        sources : typing.Optional[typing.Union[PatientTransactionSource, typing.Sequence[PatientTransactionSource]]]

        sort : typing.Optional[PatientRefundSortField]
            Defaults to refund_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefundsPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_refunds.v_1.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            limit=limit,
            patient_external_id=patient_external_id,
            claim_id=claim_id,
            service_line_id=service_line_id,
            billing_provider_id=billing_provider_id,
            unattributed=unattributed,
            invoice_id=invoice_id,
            sources=sources,
            sort=sort,
            sort_direction=sort_direction,
            page_token=page_token,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PatientRefund:
        """
        Retrieves a previously created patient refund by its `patient_refund_id`.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefund

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_refunds.v_1.get(
                patient_refund_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(patient_refund_id, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        amount_cents: int,
        patient_external_id: PatientExternalId,
        allocations: typing.Sequence[AllocationCreate],
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[str] = OMIT,
        invoice: typing.Optional[InvoiceId] = OMIT,
        refund_reason: typing.Optional[RefundReason] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientRefund:
        """
        Creates a new patient refund record and returns the newly created PatientRefund object.
        The allocations can describe whether the refund is being applied toward a specific service line,
        claim, or billing provider.

        Parameters
        ----------
        amount_cents : int

        patient_external_id : PatientExternalId

        allocations : typing.Sequence[AllocationCreate]

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[str]

        invoice : typing.Optional[InvoiceId]

        refund_reason : typing.Optional[RefundReason]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefund

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.financials import (
            AllocationCreate,
            AllocationTargetCreate_ServiceLineById,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_refunds.v_1.create(
                amount_cents=1,
                patient_external_id="patient_external_id",
                allocations=[
                    AllocationCreate(
                        amount_cents=1,
                        target=AllocationTargetCreate_ServiceLineById(
                            value=uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            )
                        ),
                    ),
                    AllocationCreate(
                        amount_cents=1,
                        target=AllocationTargetCreate_ServiceLineById(
                            value=uuid.UUID(
                                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                            )
                        ),
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            amount_cents=amount_cents,
            patient_external_id=patient_external_id,
            allocations=allocations,
            refund_timestamp=refund_timestamp,
            refund_note=refund_note,
            invoice=invoice,
            refund_reason=refund_reason,
            request_options=request_options,
        )
        return _response.data

    async def update(
        self,
        patient_refund_id: PatientRefundId,
        *,
        refund_timestamp: typing.Optional[dt.datetime] = OMIT,
        refund_note: typing.Optional[NoteUpdate] = OMIT,
        invoice: typing.Optional[InvoiceUpdate] = OMIT,
        refund_reason: typing.Optional[RefundReasonUpdate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatientRefund:
        """
        Updates the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        refund_timestamp : typing.Optional[dt.datetime]

        refund_note : typing.Optional[NoteUpdate]

        invoice : typing.Optional[InvoiceUpdate]

        refund_reason : typing.Optional[RefundReasonUpdate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatientRefund

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_refunds.v_1.update(
                patient_refund_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            patient_refund_id,
            refund_timestamp=refund_timestamp,
            refund_note=refund_note,
            invoice=invoice,
            refund_reason=refund_reason,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, patient_refund_id: PatientRefundId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes the patient refund record matching the provided patient_refund_id.

        Parameters
        ----------
        patient_refund_id : PatientRefundId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.patient_refunds.v_1.delete(
                patient_refund_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(patient_refund_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rate import Rate
from .rate_upload import RateUpload
from .validation_error import ValidationError


class RateUploadWithPossibleErrors(UniversalBaseModel):
    rate_upload: RateUpload
    existing_rate: typing.Optional[Rate] = None
    possible_errors: typing.List[ValidationError]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    InsuranceAdjustmentInfo,
    InsurancePaymentInfo,
    InvalidFiltersErrorType,
    InventoryRecord,
    InvoiceItemizationResponse,
    ListInventoryPagedResponse,
    NonInsuranceAdjustmentInfo,
    NonInsurancePaymentInfo,
    PatientAdjustmentInfo,
    PatientArStatus,
    PatientPaymentAllocation,
    PatientPaymentInfo,
    ServiceLineItemization,
)
from .errors import InvalidFiltersError

__all__ = [
    "InsuranceAdjustmentInfo",
    "InsurancePaymentInfo",
    "InvalidFiltersError",
    "InvalidFiltersErrorType",
    "InventoryRecord",
    "InvoiceItemizationResponse",
    "ListInventoryPagedResponse",
    "NonInsuranceAdjustmentInfo",
    "NonInsurancePaymentInfo",
    "PatientAdjustmentInfo",
    "PatientArStatus",
    "PatientPaymentAllocation",
    "PatientPaymentInfo",
    "ServiceLineItemization",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AttachmentInclusion,
    CreateSupplementalInformationRequest,
    SupplementalInformation,
    SupplementalInformationId,
    UpdateSupplementalInformationRequest,
)

__all__ = [
    "AttachmentInclusion",
    "CreateSupplementalInformationRequest",
    "SupplementalInformation",
    "SupplementalInformationId",
    "UpdateSupplementalInformationRequest",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.date_range_optional_end import DateRangeOptionalEnd
from .identifier_code import IdentifierCode
from .identifier_value import IdentifierValue


class IdentifierBase(UniversalBaseModel):
    period: typing.Optional[DateRangeOptionalEnd] = None
    identifier_code: IdentifierCode
    identifier_value: IdentifierValue

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AuthZeroMetadata,
    GoogleAppsMetadata,
    HumanUserMetadata,
    IdpUserMetadata,
    IdpUserMetadata_AuthZeroMetadata,
    IdpUserMetadata_GoogleAppsMetadata,
    IdpUserMetadata_OtherIdpMetadata,
    MachineUserMetadata,
    OtherIdpMetadata,
    UserMetadata,
    UserMetadata_HumanUserMetadata,
    UserMetadata_MachineUserMetadata,
    UserV2,
)

__all__ = [
    "AuthZeroMetadata",
    "GoogleAppsMetadata",
    "HumanUserMetadata",
    "IdpUserMetadata",
    "IdpUserMetadata_AuthZeroMetadata",
    "IdpUserMetadata_GoogleAppsMetadata",
    "IdpUserMetadata_OtherIdpMetadata",
    "MachineUserMetadata",
    "OtherIdpMetadata",
    "UserMetadata",
    "UserMetadata_HumanUserMetadata",
    "UserMetadata_MachineUserMetadata",
    "UserV2",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AdmittingDiagnosis,
    AdmittingDiagnosisCodeQualifier,
    AdmittingDiagnosisNew,
    ConditionCode,
    ConditionInformation,
    ConditionInformationCodeQualifier,
    ConditionInformationNew,
    D8Date,
    DiagnosisRelatedGroup,
    DiagnosisRelatedGroupCodeQualifier,
    DiagnosisRelatedGroupNew,
    ExternalCauseOfInjury,
    ExternalCauseOfInjuryCodeQualifier,
    ExternalCauseOfInjuryNew,
    HealthCareCodeInformation,
    HealthCareCodeInformationCreate,
    HealthCareCodeInformationGetAllResponse,
    HealthCareCodeInformationNew,
    HealthCareCodeInformationUpdate,
    IcdCode,
    MsdrgCode,
    OccurrenceCode,
    OccurrenceInformation,
    OccurrenceInformationCodeQualifier,
    OccurrenceInformationNew,
    OccurrenceSpanCode,
    OccurrenceSpanCodeQualifier,
    OccurrenceSpanInformation,
    OccurrenceSpanInformationNew,
    OtherDiagnosisInformation,
    OtherDiagnosisInformationCodeQualifier,
    OtherDiagnosisInformationNew,
    OtherProcedureInformation,
    OtherProcedureInformationCodeQualifier,
    OtherProcedureInformationNew,
    PatientReasonForVisit,
    PatientReasonForVisitCodeQualifier,
    PatientReasonForVisitNew,
    PresentOnAdmissionIndicatorCode,
    PrincipalDiagnosis,
    PrincipalDiagnosisCodeQualifier,
    PrincipalDiagnosisNew,
    PrincipalProcedureInformation,
    PrincipalProcedureInformationCodeQualifier,
    PrincipalProcedureInformationNew,
    Rd8Date,
    SetOrClearAdmittingDiagnosis,
    SetOrClearDiagnosisRelatedGroup,
    SetOrClearPrincipalDiagnosis,
    SetOrClearPrincipalProcedureInformation,
    TreatmentCode,
    TreatmentCodeInformation,
    TreatmentCodeInformationNew,
    TreatmentCodeQualifier,
    ValueCode,
    ValueCodeQualifier,
    ValueInformation,
    ValueInformationNew,
)

__all__ = [
    "AdmittingDiagnosis",
    "AdmittingDiagnosisCodeQualifier",
    "AdmittingDiagnosisNew",
    "ConditionCode",
    "ConditionInformation",
    "ConditionInformationCodeQualifier",
    "ConditionInformationNew",
    "D8Date",
    "DiagnosisRelatedGroup",
    "DiagnosisRelatedGroupCodeQualifier",
    "DiagnosisRelatedGroupNew",
    "ExternalCauseOfInjury",
    "ExternalCauseOfInjuryCodeQualifier",
    "ExternalCauseOfInjuryNew",
    "HealthCareCodeInformation",
    "HealthCareCodeInformationCreate",
    "HealthCareCodeInformationGetAllResponse",
    "HealthCareCodeInformationNew",
    "HealthCareCodeInformationUpdate",
    "IcdCode",
    "MsdrgCode",
    "OccurrenceCode",
    "OccurrenceInformation",
    "OccurrenceInformationCodeQualifier",
    "OccurrenceInformationNew",
    "OccurrenceSpanCode",
    "OccurrenceSpanCodeQualifier",
    "OccurrenceSpanInformation",
    "OccurrenceSpanInformationNew",
    "OtherDiagnosisInformation",
    "OtherDiagnosisInformationCodeQualifier",
    "OtherDiagnosisInformationNew",
    "OtherProcedureInformation",
    "OtherProcedureInformationCodeQualifier",
    "OtherProcedureInformationNew",
    "PatientReasonForVisit",
    "PatientReasonForVisitCodeQualifier",
    "PatientReasonForVisitNew",
    "PresentOnAdmissionIndicatorCode",
    "PrincipalDiagnosis",
    "PrincipalDiagnosisCodeQualifier",
    "PrincipalDiagnosisNew",
    "PrincipalProcedureInformation",
    "PrincipalProcedureInformationCodeQualifier",
    "PrincipalProcedureInformationNew",
    "Rd8Date",
    "SetOrClearAdmittingDiagnosis",
    "SetOrClearDiagnosisRelatedGroup",
    "SetOrClearPrincipalDiagnosis",
    "SetOrClearPrincipalProcedureInformation",
    "TreatmentCode",
    "TreatmentCodeInformation",
    "TreatmentCodeInformationNew",
    "TreatmentCodeQualifier",
    "ValueCode",
    "ValueCodeQualifier",
    "ValueInformation",
    "ValueInformationNew",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.claim_id import ClaimId
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .....encounter_providers.resources.v_2.types.ordering_provider import OrderingProvider
from .drug_identification import DrugIdentification
from .service_line_denial_reason import ServiceLineDenialReason
from .test_result import TestResult


class ServiceLineCreateStandaloneBaseBase(UniversalBaseModel):
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    charge_amount_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount charged for this service line, factoring in quantity. The system will attempt to set it based on the chargemaster configuration and the service line’s quantity. For example, if a single unit has an entry of 100 cents and 2 units were rendered, the charge_amount_cents will be set to 200. If there is no chargemaster entry, the system will default to the amount set via this field.
    """

    denial_reason: typing.Optional[ServiceLineDenialReason] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    837p Loop2300, SV105. This enum is not used or required in 837i claims. If your organization does not intend to submit claims with a different place of service at the service line level, this field should not be populated. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    quantity: Decimal = pydantic.Field()
    """
    String representation of a Decimal that can be parsed by most libraries.
    For professional claims, a ServiceLine quantity cannot contain more than one digit of precision
    (Example: 1.1 is valid, 1.11 is not). For institutional claims, a ServiceLine quantity cannot contain
    more than three decimal digits of precision.
    """

    units: ServiceLineUnits
    claim_id: ClaimId
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on a 837-P and SV2-02, C003-07 on a 837-I form.
    """

    date_of_service: typing.Optional[dt.date] = None
    end_date_of_service: typing.Optional[dt.date] = None
    drug_identification: typing.Optional[DrugIdentification] = None
    ordering_provider: typing.Optional[OrderingProvider] = pydantic.Field(default=None)
    """
    Required when the service or supply was ordered by a provider who is different than the rendering provider for this service line.
    If not required by this implementation guide, do not send.
    """

    test_results: typing.Optional[typing.List[TestResult]] = pydantic.Field(default=None)
    """
    Contains a list of test results. Test result types may map to MEA-02 on the 837-P (ex: Hemoglobin, Hematocrit).
    This is unused by 837-i and ignored for institutional service lines.
    No more than 5 MEA-02 test results may be submitted per service line.
    """

    has_epsdt_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-11 on the 837-P and Box 24H on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    has_family_planning_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-12 on the 837-P and Box 24I on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Maps to NTE02 loop 2400 on the EDI 837.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import AppointmentListItem, AppointmentListPage, PatientListItem, PatientListPage, SortFieldString

__all__ = ["AppointmentListItem", "AppointmentListPage", "PatientListItem", "PatientListPage", "SortFieldString"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .clinical_trial_phase import ClinicalTrialPhase


class MutableClinicalTrial(UniversalBaseModel):
    name: str
    clinical_trial_number: str = pydantic.Field()
    """
    Must be 8 characters long
    """

    clinical_trial_phase: typing.Optional[ClinicalTrialPhase] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .auth_zero_metadata import AuthZeroMetadata
from .google_apps_metadata import GoogleAppsMetadata
from .human_user_metadata import HumanUserMetadata
from .idp_user_metadata import (
    IdpUserMetadata,
    IdpUserMetadata_AuthZeroMetadata,
    IdpUserMetadata_GoogleAppsMetadata,
    IdpUserMetadata_OtherIdpMetadata,
)
from .machine_user_metadata import MachineUserMetadata
from .other_idp_metadata import OtherIdpMetadata
from .user_metadata import UserMetadata, UserMetadata_HumanUserMetadata, UserMetadata_MachineUserMetadata
from .user_v_2 import UserV2

__all__ = [
    "AuthZeroMetadata",
    "GoogleAppsMetadata",
    "HumanUserMetadata",
    "IdpUserMetadata",
    "IdpUserMetadata_AuthZeroMetadata",
    "IdpUserMetadata_GoogleAppsMetadata",
    "IdpUserMetadata_OtherIdpMetadata",
    "MachineUserMetadata",
    "OtherIdpMetadata",
    "UserMetadata",
    "UserMetadata_HumanUserMetadata",
    "UserMetadata_MachineUserMetadata",
    "UserV2",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .state import State


class StreetAddressOptionalBase(UniversalBaseModel):
    address_1: typing.Optional[str] = pydantic.Field(alias="address1", default=None)
    address_2: typing.Optional[str] = pydantic.Field(alias="address2", default=None)
    city: typing.Optional[str] = None
    state: typing.Optional[State] = None
    zip_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    5-digit zip code
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Rd8Date(UniversalBaseModel):
    start: dt.datetime = pydantic.Field()
    """
    An [RFC 3339, section 5.6 datetime](https://ijmacd.github.io/rfc3339-iso8601/).
    For example, 2017-07-21T17:32:28Z.
    In practice, only the date portion of this is used for claim submission, so
    midnight time is fine.
    """

    end: dt.datetime = pydantic.Field()
    """
    An [RFC 3339, section 5.6 datetime](https://ijmacd.github.io/rfc3339-iso8601/).
    For example, 2017-07-21T17:32:28Z.
    In practice, only the date portion of this is used for claim submission, so
    midnight time is fine.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .batch_eligibility_response import BatchEligibilityResponse
from .coordination_of_benefits_recommendation import CoordinationOfBenefitsRecommendation
from .eligibility_check import EligibilityCheck
from .eligibility_check_error import EligibilityCheckError
from .eligibility_check_error_details import EligibilityCheckErrorDetails
from .eligibility_check_error_source import EligibilityCheckErrorSource
from .eligibility_check_metadata import EligibilityCheckMetadata
from .eligibility_check_page import EligibilityCheckPage
from .eligibility_check_status import EligibilityCheckStatus
from .eligibility_recommendation import EligibilityRecommendation
from .eligibility_recommendation_patient_info import EligibilityRecommendationPatientInfo
from .eligibility_recommendation_payload import (
    EligibilityRecommendationPayload,
    EligibilityRecommendationPayload_CoordinationOfBenefits,
    EligibilityRecommendationPayload_MedicareAdvantage,
)
from .eligibility_request import EligibilityRequest
from .eligibility_response import EligibilityResponse
from .eligibility_status import EligibilityStatus
from .encounter import Encounter
from .individual_provider import IndividualProvider
from .medicare_advantage_recommendation import MedicareAdvantageRecommendation
from .medicare_advantage_recommendation_payload import MedicareAdvantageRecommendationPayload
from .organization_provider import OrganizationProvider
from .parsed_response import ParsedResponse
from .payer_search_response import PayerSearchResponse
from .post_eligibility_recommendation_request import PostEligibilityRecommendationRequest
from .provider import Provider
from .request_correction import RequestCorrection
from .stedi_payer import StediPayer
from .stedi_payer_item import StediPayerItem

__all__ = [
    "BatchEligibilityResponse",
    "CoordinationOfBenefitsRecommendation",
    "EligibilityCheck",
    "EligibilityCheckError",
    "EligibilityCheckErrorDetails",
    "EligibilityCheckErrorSource",
    "EligibilityCheckMetadata",
    "EligibilityCheckPage",
    "EligibilityCheckStatus",
    "EligibilityRecommendation",
    "EligibilityRecommendationPatientInfo",
    "EligibilityRecommendationPayload",
    "EligibilityRecommendationPayload_CoordinationOfBenefits",
    "EligibilityRecommendationPayload_MedicareAdvantage",
    "EligibilityRequest",
    "EligibilityResponse",
    "EligibilityStatus",
    "Encounter",
    "IndividualProvider",
    "MedicareAdvantageRecommendation",
    "MedicareAdvantageRecommendationPayload",
    "OrganizationProvider",
    "ParsedResponse",
    "PayerSearchResponse",
    "PostEligibilityRecommendationRequest",
    "Provider",
    "RequestCorrection",
    "StediPayer",
    "StediPayerItem",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .insurance_card import InsuranceCard
from .insurance_card_base import InsuranceCardBase
from .insurance_card_create import InsuranceCardCreate
from .insurance_card_create_optional import InsuranceCardCreateOptional
from .insurance_card_id import InsuranceCardId

__all__ = [
    "InsuranceCard",
    "InsuranceCardBase",
    "InsuranceCardCreate",
    "InsuranceCardCreateOptional",
    "InsuranceCardId",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.service_line_id import ServiceLineId
from .claim_invoice_item import ClaimInvoiceItem
from .service_line_invoice_item import ServiceLineInvoiceItem


class ClaimInvoiceItemInfo(UniversalBaseModel):
    claim_invoice_item: typing.Optional[ClaimInvoiceItem] = None
    service_line_invoice_items: typing.Dict[ServiceLineId, ServiceLineInvoiceItem]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....billing_notes.resources.v_2.types.billing_note_base_optional import BillingNoteBaseOptional
from .....claim_submission.resources.v_1.types.external_claim_submission_create_optional import (
    ExternalClaimSubmissionCreateOptional,
)
from .....commons.types.street_address_short_zip_optional import StreetAddressShortZipOptional
from .....custom_schemas.resources.v_1.types.schema_instance_optional import SchemaInstanceOptional
from .....diagnoses.types.diagnosis_create_optional import DiagnosisCreateOptional
from .....encounter_providers.resources.v_2.types.billing_provider_update_with_optional_address import (
    BillingProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.initial_referring_provider_update_with_optional_address import (
    InitialReferringProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.referring_provider_update_with_optional_address import (
    ReferringProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.rendering_provider_update_with_optional_address import (
    RenderingProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.supervising_provider_update_with_optional_address import (
    SupervisingProviderUpdateWithOptionalAddress,
)
from .....guarantor.resources.v_1.types.guarantor_optional import GuarantorOptional
from .....individual.types.patient_update_with_optional_address import PatientUpdateWithOptionalAddress
from .....individual.types.subscriber_create_optional import SubscriberCreateOptional
from .....service_facility.types.encounter_service_facility_update_with_optional_address import (
    EncounterServiceFacilityUpdateWithOptionalAddress,
)
from .....service_lines.resources.v_2.types.service_line_create_optional import ServiceLineCreateOptional
from .claim_supplemental_information_optional import ClaimSupplementalInformationOptional
from .clinical_note_category_create_optional import ClinicalNoteCategoryCreateOptional
from .encounter_optional import EncounterOptional
from .epsdt_referral_optional import EpsdtReferralOptional
from .intervention_optional import InterventionOptional
from .medication_optional import MedicationOptional
from .patient_history_category_optional import PatientHistoryCategoryOptional
from .vitals_update import VitalsUpdate


class EncounterDeepOptional(EncounterOptional):
    vitals: typing.Optional[VitalsUpdate] = pydantic.Field(default=None)
    """
    If a vitals entity already exists for the encounter, then all values will be updated to the provided values.
    Otherwise, a new vitals object will be created for the encounter.
    """

    diagnoses: typing.Optional[typing.List[DiagnosisCreateOptional]] = pydantic.Field(default=None)
    """
    Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
    may be submitted at this time, and coders will later prioritize the 12 that will be
    submitted to the payor.
    """

    clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreateOptional]] = pydantic.Field(default=None)
    """
    Holds a collection of clinical observations made by healthcare providers during patient encounters. Please note that medical records for appeals should be sent using the Encounter Attachments API.
    """

    claim_supplemental_information: typing.Optional[typing.List[ClaimSupplementalInformationOptional]] = pydantic.Field(
        default=None
    )
    """
    Refers to Loop 2300 - Segment PWK on the 837P and 837i form. No more than 10 entries are permitted.
    """

    epsdt_referral: typing.Optional[EpsdtReferralOptional] = pydantic.Field(default=None)
    """
    Refers to Box 24H on the CMS1500 form and Loop 2300 CRC - EPSDT Referral on the 837P and 837i form
    """

    existing_medications: typing.Optional[typing.List[MedicationOptional]] = pydantic.Field(default=None)
    """
    Existing medications that should be on the encounter.
    Note all current existing medications on encounter will be overridden with this list.
    """

    guarantor: typing.Optional[GuarantorOptional] = pydantic.Field(default=None)
    """
    Personal and contact info for the guarantor of the patient responsibility.
    """

    subscriber_primary: typing.Optional[SubscriberCreateOptional] = pydantic.Field(default=None)
    """
    Contains details of the primary insurance subscriber.
    """

    subscriber_secondary: typing.Optional[SubscriberCreateOptional] = pydantic.Field(default=None)
    """
    Contains details of the secondary insurance subscriber.
    """

    subscriber_tertiary: typing.Optional[SubscriberCreateOptional] = pydantic.Field(default=None)
    """
    Contains details of the tertiary insurance subscriber.
    """

    interventions: typing.Optional[typing.List[InterventionOptional]] = None
    schema_instances: typing.Optional[typing.List[SchemaInstanceOptional]] = pydantic.Field(default=None)
    """
    Key-value pairs that must adhere to a schema created via the Custom Schema API. Multiple schema
    instances cannot be created for the same schema on an encounter. Updating schema instances utilizes PUT
    semantics, so the schema instances on the encounter will be set to whatever inputs are provided. If null
    is provided as an input, then the encounter's schema instances will be cleared.
    """

    external_claim_submission: typing.Optional[ExternalClaimSubmissionCreateOptional] = pydantic.Field(default=None)
    """
    ***This field is in beta.***
    To be included for claims that have been submitted outside of Candid.
    Candid supports posting remits and payments to these claims and working them in-platform (e.g. editing, resubmitting).
    """

    service_lines: typing.Optional[typing.List[ServiceLineCreateOptional]] = pydantic.Field(default=None)
    """
    Each service line must be linked to a diagnosis. Concretely,
    `service_line.diagnosis_pointers`must contain at least one entry which should be
    in bounds of the diagnoses list field.
    """

    patient_histories: typing.Optional[typing.List[PatientHistoryCategoryOptional]] = None
    billing_notes: typing.Optional[typing.List[BillingNoteBaseOptional]] = pydantic.Field(default=None)
    """
    Spot to store misc, human-readable, notes about this encounter to be
    used in the billing process.
    """

    patient: typing.Optional[PatientUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    Contains the identification information of the individual receiving medical services.
    """

    service_facility: typing.Optional[EncounterServiceFacilityUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. There is no equivalent on the paper UB-04 claim form, but this field is equivalent to Loop 2310E Service Facility Location details on an 837i form, and is used when this is different to the entity identified as the Billing Provider. Note that for an in-network claim to be successfully adjudicated, the service facility address listed
    """

    rendering_provider: typing.Optional[RenderingProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service. For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.
    """

    initial_referring_provider: typing.Optional[InitialReferringProviderUpdateWithOptionalAddress] = pydantic.Field(
        default=None
    )
    """
    The second iteration of Loop ID-2310. Use code "P3 - Primary Care Provider" in this loop to
    indicate the initial referral from the primary care provider or whatever provider wrote the initial referral for this patient's episode of care being billed/reported in this transaction.
    """

    referring_provider: typing.Optional[ReferringProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    The final provider who referred the services that were rendered.
    All physicians who order services or refer Medicare beneficiaries must
    report this data.
    """

    supervising_provider: typing.Optional[SupervisingProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    Required when the rendering provider is supervised by a physician. If not required by this implementation guide, do not send.
    """

    billing_provider: typing.Optional[BillingProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim form or Form Locator 1 on a UB-04 claim form.
    """

    pay_to_address: typing.Optional[StreetAddressShortZipOptional] = pydantic.Field(default=None)
    """
    Specifies the address to which payments for the claim should be sent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.api_error import ApiError
from ..types.entity_not_found_error_message import EntityNotFoundErrorMessage


class EntityNotFoundError(ApiError):
    def __init__(self, body: EntityNotFoundErrorMessage, headers: typing.Optional[typing.Dict[str, str]] = None):
        super().__init__(status_code=404, headers=headers, body=body)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CardPaymentMethod,
    CashPaymentMethod,
    CheckPaymentMethod,
    PatientPayment,
    PatientPaymentId,
    PatientPaymentSortField,
    PatientPaymentsPage,
    PaymentMethod,
    PaymentMethodDetail,
    PaymentMethod_Card,
    PaymentMethod_Cash,
    PaymentMethod_Check,
)

__all__ = [
    "CardPaymentMethod",
    "CashPaymentMethod",
    "CheckPaymentMethod",
    "PatientPayment",
    "PatientPaymentId",
    "PatientPaymentSortField",
    "PatientPaymentsPage",
    "PaymentMethod",
    "PaymentMethodDetail",
    "PaymentMethod_Card",
    "PaymentMethod_Cash",
    "PaymentMethod_Check",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .eligibility_check_error_details import EligibilityCheckErrorDetails
from .eligibility_check_error_source import EligibilityCheckErrorSource


class EligibilityCheckError(UniversalBaseModel):
    source: EligibilityCheckErrorSource
    error_details: EligibilityCheckErrorDetails = pydantic.Field(alias="errorDetails")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .service_facility_mode import ServiceFacilityMode
from .service_facility_operational_status import ServiceFacilityOperationalStatus
from .service_facility_physical_type import ServiceFacilityPhysicalType
from .service_facility_status import ServiceFacilityStatus
from .service_facility_type import ServiceFacilityType


class OrganizationServiceFacilityCreate(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.commons import State, StreetAddressLongZip
    from candid.resources.organization_service_facilities.resources.v_2 import (
        OrganizationServiceFacilityCreate,
        ServiceFacilityMode,
        ServiceFacilityOperationalStatus,
        ServiceFacilityPhysicalType,
        ServiceFacilityStatus,
        ServiceFacilityType,
    )

    OrganizationServiceFacilityCreate(
        name="Test Service Facility",
        aliases=["Test Service Facility Alias"],
        description="Test Service Facility Description",
        status=ServiceFacilityStatus.ACTIVE,
        operational_status=ServiceFacilityOperationalStatus.CLOSED,
        mode=ServiceFacilityMode.INSTANCE,
        type=ServiceFacilityType.DIAGNOSTICS_OR_THERAPEUTICS_UNIT,
        physical_type=ServiceFacilityPhysicalType.SITE,
        telecoms=["555-555-5555"],
        address=StreetAddressLongZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
    )
    """

    name: str = pydantic.Field()
    """
    The name of the service facility.
    """

    aliases: typing.List[str] = pydantic.Field()
    """
    A list of alternate names for the service facility.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description of the service facility.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An ID for this service facility used in an external system (e.g. your EMR). Service facilities can be queried by this field.
    """

    npi: typing.Optional[str] = pydantic.Field(default=None)
    """
    An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.
    Box 32 section (a) of the CMS-1500 claim form.
    """

    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    The Place of Service (POS) code for this service facility.
    """

    clia_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The associated CLIA number with this service facility. Can be used to populate CLIA numbers on encounters.
    """

    mammography_certification_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The associated mammography certification number for this service facility. This is a 6 digit code assigned by the FDA.
    """

    status: typing.Optional[ServiceFacilityStatus] = pydantic.Field(default=None)
    """
    The status of the service facility.
    """

    operational_status: typing.Optional[ServiceFacilityOperationalStatus] = pydantic.Field(default=None)
    """
    The operational status of the service facility.
    """

    mode: typing.Optional[ServiceFacilityMode] = pydantic.Field(default=None)
    """
    The mode of the service facility.
    """

    type: typing.Optional[ServiceFacilityType] = pydantic.Field(default=None)
    """
    The type of the service facility.
    """

    physical_type: typing.Optional[ServiceFacilityPhysicalType] = pydantic.Field(default=None)
    """
    The physical type of the service facility.
    """

    telecoms: typing.List[str] = pydantic.Field()
    """
    A list of contact methods for the service facility.
    """

    address: StreetAddressLongZip = pydantic.Field()
    """
    The address of the service facility.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .era import Era
from .era_base import EraBase
from .era_id import EraId

__all__ = ["Era", "EraBase", "EraId"]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .clearinghouse_payer_info import ClearinghousePayerInfo
from .payer import Payer
from .payer_ids import PayerIds
from .payer_page import PayerPage
from .payer_uuid import PayerUuid
from .primary_payer_ids import PrimaryPayerIds
from .support_state import SupportState

__all__ = ["ClearinghousePayerInfo", "Payer", "PayerIds", "PayerPage", "PayerUuid", "PrimaryPayerIds", "SupportState"]



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawAuthClient, RawAuthClient
from .resources.default.client import AsyncDefaultClient, DefaultClient


class AuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthClient(client_wrapper=client_wrapper)
        self.default = DefaultClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthClient
        """
        return self._raw_client


class AsyncAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthClient(client_wrapper=client_wrapper)
        self.default = AsyncDefaultClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthClient
        """
        return self._raw_client



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .eligibility_recommendation_patient_info import EligibilityRecommendationPatientInfo
from .eligibility_recommendation_payload import EligibilityRecommendationPayload


class PostEligibilityRecommendationRequest(UniversalBaseModel):
    eligibility_check_id: str
    patient: EligibilityRecommendationPatientInfo
    recommendation: EligibilityRecommendationPayload

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .carc import Carc
from .claim_adjustment_reason_code import ClaimAdjustmentReasonCode
from .patient_discharge_status_code import PatientDischargeStatusCode
from .point_of_origin_for_admission_or_visit_code import PointOfOriginForAdmissionOrVisitCode
from .rarc import Rarc
from .remittance_advice_remark_code import RemittanceAdviceRemarkCode
from .type_of_admission_or_visit_code import TypeOfAdmissionOrVisitCode
from .type_of_bill_composite import TypeOfBillComposite
from .type_of_bill_composite_base import TypeOfBillCompositeBase
from .type_of_bill_composite_update import (
    TypeOfBillCompositeUpdate,
    TypeOfBillCompositeUpdate_CompositeCodes,
    TypeOfBillCompositeUpdate_RawCode,
)
from .type_of_bill_frequency_code import TypeOfBillFrequencyCode
from .type_of_care_code import TypeOfCareCode
from .type_of_facility_code import TypeOfFacilityCode

__all__ = [
    "Carc",
    "ClaimAdjustmentReasonCode",
    "PatientDischargeStatusCode",
    "PointOfOriginForAdmissionOrVisitCode",
    "Rarc",
    "RemittanceAdviceRemarkCode",
    "TypeOfAdmissionOrVisitCode",
    "TypeOfBillComposite",
    "TypeOfBillCompositeBase",
    "TypeOfBillCompositeUpdate",
    "TypeOfBillCompositeUpdate_CompositeCodes",
    "TypeOfBillCompositeUpdate_RawCode",
    "TypeOfBillFrequencyCode",
    "TypeOfCareCode",
    "TypeOfFacilityCode",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .coverage_details import CoverageDetails
from .service_coverage_details import ServiceCoverageDetails
from .service_type_code import ServiceTypeCode


class ServiceCoverage(UniversalBaseModel):
    service_code: ServiceTypeCode
    in_network: typing.Optional[ServiceCoverageDetails] = None
    in_network_flat: typing.Optional[typing.List[CoverageDetails]] = None
    out_of_network: typing.Optional[ServiceCoverageDetails] = None
    out_of_network_flat: typing.Optional[typing.List[CoverageDetails]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .page_token import PageToken


class ResourcePage(UniversalBaseModel):
    next_page_token: typing.Optional[PageToken] = None
    prev_page_token: typing.Optional[PageToken] = None
    total: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.address import Address
from .....common.types.contact_point import ContactPoint
from .....common.types.human_name import HumanName
from .....common.types.period import Period
from .....common.types.relationship import Relationship


class Contact(UniversalBaseModel):
    relationship: typing.List[Relationship]
    name: HumanName
    telecoms: typing.List[ContactPoint]
    addresses: typing.List[Address]
    period: typing.Optional[Period] = None
    hipaa_authorization: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rx_cui import RxCui


class Medication(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.resources.v_4 import Medication

    Medication(
        name="Lisinopril",
        rx_cui="860975",
        dosage="10mg",
        dosage_form="Tablet",
        frequency="Once Daily",
        as_needed=True,
    )
    """

    name: str
    rx_cui: typing.Optional[RxCui] = None
    dosage: typing.Optional[str] = None
    dosage_form: typing.Optional[str] = None
    frequency: typing.Optional[str] = None
    as_needed: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .admitting_diagnosis import AdmittingDiagnosis
from .admitting_diagnosis_code_qualifier import AdmittingDiagnosisCodeQualifier
from .admitting_diagnosis_new import AdmittingDiagnosisNew
from .condition_code import ConditionCode
from .condition_information import ConditionInformation
from .condition_information_code_qualifier import ConditionInformationCodeQualifier
from .condition_information_new import ConditionInformationNew
from .d_8_date import D8Date
from .diagnosis_related_group import DiagnosisRelatedGroup
from .diagnosis_related_group_code_qualifier import DiagnosisRelatedGroupCodeQualifier
from .diagnosis_related_group_new import DiagnosisRelatedGroupNew
from .external_cause_of_injury import ExternalCauseOfInjury
from .external_cause_of_injury_code_qualifier import ExternalCauseOfInjuryCodeQualifier
from .external_cause_of_injury_new import ExternalCauseOfInjuryNew
from .health_care_code_information import HealthCareCodeInformation
from .health_care_code_information_create import HealthCareCodeInformationCreate
from .health_care_code_information_get_all_response import HealthCareCodeInformationGetAllResponse
from .health_care_code_information_new import HealthCareCodeInformationNew
from .health_care_code_information_update import HealthCareCodeInformationUpdate
from .icd_code import IcdCode
from .msdrg_code import MsdrgCode
from .occurrence_code import OccurrenceCode
from .occurrence_information import OccurrenceInformation
from .occurrence_information_code_qualifier import OccurrenceInformationCodeQualifier
from .occurrence_information_new import OccurrenceInformationNew
from .occurrence_span_code import OccurrenceSpanCode
from .occurrence_span_code_qualifier import OccurrenceSpanCodeQualifier
from .occurrence_span_information import OccurrenceSpanInformation
from .occurrence_span_information_new import OccurrenceSpanInformationNew
from .other_diagnosis_information import OtherDiagnosisInformation
from .other_diagnosis_information_code_qualifier import OtherDiagnosisInformationCodeQualifier
from .other_diagnosis_information_new import OtherDiagnosisInformationNew
from .other_procedure_information import OtherProcedureInformation
from .other_procedure_information_code_qualifier import OtherProcedureInformationCodeQualifier
from .other_procedure_information_new import OtherProcedureInformationNew
from .patient_reason_for_visit import PatientReasonForVisit
from .patient_reason_for_visit_code_qualifier import PatientReasonForVisitCodeQualifier
from .patient_reason_for_visit_new import PatientReasonForVisitNew
from .present_on_admission_indicator_code import PresentOnAdmissionIndicatorCode
from .principal_diagnosis import PrincipalDiagnosis
from .principal_diagnosis_code_qualifier import PrincipalDiagnosisCodeQualifier
from .principal_diagnosis_new import PrincipalDiagnosisNew
from .principal_procedure_information import PrincipalProcedureInformation
from .principal_procedure_information_code_qualifier import PrincipalProcedureInformationCodeQualifier
from .principal_procedure_information_new import PrincipalProcedureInformationNew
from .rd_8_date import Rd8Date
from .set_or_clear_admitting_diagnosis import SetOrClearAdmittingDiagnosis
from .set_or_clear_diagnosis_related_group import SetOrClearDiagnosisRelatedGroup
from .set_or_clear_principal_diagnosis import SetOrClearPrincipalDiagnosis
from .set_or_clear_principal_procedure_information import SetOrClearPrincipalProcedureInformation
from .treatment_code import TreatmentCode
from .treatment_code_information import TreatmentCodeInformation
from .treatment_code_information_new import TreatmentCodeInformationNew
from .treatment_code_qualifier import TreatmentCodeQualifier
from .value_code import ValueCode
from .value_code_qualifier import ValueCodeQualifier
from .value_information import ValueInformation
from .value_information_new import ValueInformationNew

__all__ = [
    "AdmittingDiagnosis",
    "AdmittingDiagnosisCodeQualifier",
    "AdmittingDiagnosisNew",
    "ConditionCode",
    "ConditionInformation",
    "ConditionInformationCodeQualifier",
    "ConditionInformationNew",
    "D8Date",
    "DiagnosisRelatedGroup",
    "DiagnosisRelatedGroupCodeQualifier",
    "DiagnosisRelatedGroupNew",
    "ExternalCauseOfInjury",
    "ExternalCauseOfInjuryCodeQualifier",
    "ExternalCauseOfInjuryNew",
    "HealthCareCodeInformation",
    "HealthCareCodeInformationCreate",
    "HealthCareCodeInformationGetAllResponse",
    "HealthCareCodeInformationNew",
    "HealthCareCodeInformationUpdate",
    "IcdCode",
    "MsdrgCode",
    "OccurrenceCode",
    "OccurrenceInformation",
    "OccurrenceInformationCodeQualifier",
    "OccurrenceInformationNew",
    "OccurrenceSpanCode",
    "OccurrenceSpanCodeQualifier",
    "OccurrenceSpanInformation",
    "OccurrenceSpanInformationNew",
    "OtherDiagnosisInformation",
    "OtherDiagnosisInformationCodeQualifier",
    "OtherDiagnosisInformationNew",
    "OtherProcedureInformation",
    "OtherProcedureInformationCodeQualifier",
    "OtherProcedureInformationNew",
    "PatientReasonForVisit",
    "PatientReasonForVisitCodeQualifier",
    "PatientReasonForVisitNew",
    "PresentOnAdmissionIndicatorCode",
    "PrincipalDiagnosis",
    "PrincipalDiagnosisCodeQualifier",
    "PrincipalDiagnosisNew",
    "PrincipalProcedureInformation",
    "PrincipalProcedureInformationCodeQualifier",
    "PrincipalProcedureInformationNew",
    "Rd8Date",
    "SetOrClearAdmittingDiagnosis",
    "SetOrClearDiagnosisRelatedGroup",
    "SetOrClearPrincipalDiagnosis",
    "SetOrClearPrincipalProcedureInformation",
    "TreatmentCode",
    "TreatmentCodeInformation",
    "TreatmentCodeInformationNew",
    "TreatmentCodeQualifier",
    "ValueCode",
    "ValueCodeQualifier",
    "ValueInformation",
    "ValueInformationNew",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .address_type import AddressType


class OrganizationProviderAddress(UniversalBaseModel):
    address: StreetAddressLongZip = pydantic.Field()
    """
    The address of the provider
    """

    address_type: AddressType = pydantic.Field()
    """
    The address type of the provider
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    DenialReasonContent,
    DrugIdentification,
    DrugIdentificationOptional,
    MeasurementUnitCode,
    ServiceIdQualifier,
    ServiceLine,
    ServiceLineAdjustment,
    ServiceLineCreate,
    ServiceLineCreateBase,
    ServiceLineCreateBaseBase,
    ServiceLineCreateOptional,
    ServiceLineCreateStandalone,
    ServiceLineCreateStandaloneBase,
    ServiceLineCreateStandaloneBaseBase,
    ServiceLineDenialReason,
    ServiceLineEraData,
    ServiceLineUpdate,
    ServiceLineUpdateBase,
    TestResult,
    TestResultOptional,
    TestResultType,
    UniversalServiceLineCreate,
    UniversalServiceLineCreateStandalone,
    UniversalServiceLineUpdate,
)

__all__ = [
    "DenialReasonContent",
    "DrugIdentification",
    "DrugIdentificationOptional",
    "MeasurementUnitCode",
    "ServiceIdQualifier",
    "ServiceLine",
    "ServiceLineAdjustment",
    "ServiceLineCreate",
    "ServiceLineCreateBase",
    "ServiceLineCreateBaseBase",
    "ServiceLineCreateOptional",
    "ServiceLineCreateStandalone",
    "ServiceLineCreateStandaloneBase",
    "ServiceLineCreateStandaloneBaseBase",
    "ServiceLineDenialReason",
    "ServiceLineEraData",
    "ServiceLineUpdate",
    "ServiceLineUpdateBase",
    "TestResult",
    "TestResultOptional",
    "TestResultType",
    "UniversalServiceLineCreate",
    "UniversalServiceLineCreateStandalone",
    "UniversalServiceLineUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.address import Address
from .....common.types.canonical_non_insurance_payer_association import CanonicalNonInsurancePayerAssociation
from .....common.types.canonical_non_insurance_payer_id import CanonicalNonInsurancePayerId
from .....common.types.contact_point import ContactPoint
from .....common.types.disability_status import DisabilityStatus
from .....common.types.ethnicity import Ethnicity
from .....common.types.external_identifier import ExternalIdentifier
from .....common.types.external_provider import ExternalProvider
from .....common.types.gender import Gender
from .....common.types.human_name import HumanName
from .....common.types.note_id import NoteId
from .....common.types.patient_service_facility import PatientServiceFacility
from .....common.types.race import Race
from .....common.types.sex import Sex
from .....common.types.sexual_orientation import SexualOrientation
from .....common.types.tag_id import TagId
from .authorization import Authorization
from .contact import Contact
from .do_not_invoice_reason import DoNotInvoiceReason
from .external_provenance import ExternalProvenance
from .filing_order import FilingOrder
from .guarantor import Guarantor
from .inferred_patient_metadata import InferredPatientMetadata
from .marital_status import MaritalStatus
from .origination_detail import OriginationDetail
from .referral import Referral


class MutablePatient(UniversalBaseModel):
    """
    An object representing patient demographics information.
    """

    name: HumanName
    other_names: typing.List[HumanName] = pydantic.Field()
    """
    Other names for the patient.
    """

    other_identifiers: typing.Optional[typing.List[ExternalIdentifier]] = pydantic.Field(default=None)
    """
    Other identifiers for the patient.
    """

    gender: typing.Optional[Gender] = None
    birth_date: dt.date
    social_security_number: typing.Optional[str] = None
    biological_sex: Sex = pydantic.Field()
    """
    The biological sex of the patient. This corresponds to the HL7 AdministrativeGender https://www.hl7.org/fhir/valueset-administrative-gender.html
    """

    sexual_orientation: typing.Optional[SexualOrientation] = pydantic.Field(default=None)
    """
    The sexual orientation of the patient.
    """

    pronouns: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The pronouns of the patient.
    """

    race: typing.Optional[Race] = None
    ethnicity: typing.Optional[Ethnicity] = None
    disability_status: typing.Optional[DisabilityStatus] = None
    marital_status: typing.Optional[MaritalStatus] = None
    deceased: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Time of death for the patient.  Leave unset if the patient is not deceased.
    """

    multiple_birth: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of siblings the patient was born with.  Leave unset if the patient was not part of a multiple birth.
    """

    primary_address: Address = pydantic.Field()
    """
    The primary address for the patient.
    """

    other_addresses: typing.List[Address] = pydantic.Field()
    """
    Other addresses for the patient.
    """

    primary_telecom: typing.Optional[ContactPoint] = pydantic.Field(default=None)
    """
    The primary phone number for the patient.
    """

    other_telecoms: typing.List[ContactPoint] = pydantic.Field()
    """
    Other phone numbers for the patient.
    """

    email: typing.Optional[str] = None
    electronic_communication_opt_in: typing.Optional[bool] = None
    photo: typing.Optional[str] = None
    language: typing.Optional[str] = None
    external_provenance: typing.Optional[ExternalProvenance] = pydantic.Field(default=None)
    """
    Information about the upstream system that owns this patient data.  Leave unset if Candid owns patient data.
    """

    contacts: typing.List[Contact] = pydantic.Field()
    """
    Contacts for the patient.
    """

    general_practitioners: typing.List[ExternalProvider]
    filing_order: FilingOrder
    non_insurance_payers: typing.Optional[typing.List[CanonicalNonInsurancePayerId]] = None
    non_insurance_payer_associations: typing.Optional[typing.List[CanonicalNonInsurancePayerAssociation]] = None
    guarantor: typing.Optional[Guarantor] = None
    self_pay: typing.Optional[bool] = None
    authorizations: typing.Optional[typing.List[Authorization]] = None
    referrals: typing.Optional[typing.List[Referral]] = None
    primary_service_facility_id: typing.Optional[str] = None
    service_facilities: typing.Optional[typing.List[PatientServiceFacility]] = pydantic.Field(default=None)
    """
    Associated service facilities for this patient.
    """

    do_not_invoice_reason: typing.Optional[DoNotInvoiceReason] = pydantic.Field(default=None)
    """
    If this value is defined, the customer will not be invoiced.
    """

    note_ids: typing.Optional[typing.List[NoteId]] = None
    tag_ids: typing.Optional[typing.List[TagId]] = None
    origination_detail: typing.Optional[OriginationDetail] = pydantic.Field(default=None)
    """
    Information about the patient source, if applicable.
    """

    inferred_patient_metadata: typing.Optional[InferredPatientMetadata] = pydantic.Field(default=None)
    """
    Metadata for the patient used for patient inference from encounters.
    """

    orcon: typing.Optional[bool] = pydantic.Field(default=None)
    """
    ORCON (Originator Controlled) - When set to true, the Candid system will hide this patient from downstream integrations. Defaults to false.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.invoice_id import InvoiceId
from ....commons.types.page_token import PageToken
from ....commons.types.sort_direction import SortDirection
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....financials.types.invoice_update import InvoiceUpdate
from ....financials.types.note_update import NoteUpdate
from ....non_insurance_payers.resources.v_1.types.non_insurance_payer_id import NonInsurancePayerId
from .types.non_insurance_payer_payment import NonInsurancePayerPayment
from .types.non_insurance_payer_payment_create import NonInsurancePayerPaymentCreate
from .types.non_insurance_payer_payment_id import NonInsurancePayerPaymentId
from .types.non_insurance_payer_payment_sort_field import NonInsurancePayerPaymentSortField
from .types.non_insurance_payer_payments_page import NonInsurancePayerPaymentsPage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        non_insurance_payer_id: typing.Optional[NonInsurancePayerId] = None,
        check_number: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sort: typing.Optional[NonInsurancePayerPaymentSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[NonInsurancePayerPaymentsPage]:
        """
        Returns all non-insurance payer payments

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        non_insurance_payer_id : typing.Optional[NonInsurancePayerId]

        check_number : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        invoice_id : typing.Optional[InvoiceId]

        sort : typing.Optional[NonInsurancePayerPaymentSortField]
            Defaults to refund_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[NonInsurancePayerPaymentsPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/non-insurance-payer-payments/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "non_insurance_payer_id": non_insurance_payer_id,
                "check_number": check_number,
                "invoice_id": invoice_id,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPaymentsPage,
                parse_obj_as(
                    type_=NonInsurancePayerPaymentsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        non_insurance_payer_payment_id: NonInsurancePayerPaymentId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[NonInsurancePayerPayment]:
        """
        Retrieves a previously created non-insurance payer payment by its `non_insurance_payer_payment_id`.

        Parameters
        ----------
        non_insurance_payer_payment_id : NonInsurancePayerPaymentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[NonInsurancePayerPayment]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payer-payments/v1/{jsonable_encoder(non_insurance_payer_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPayment,
                parse_obj_as(
                    type_=NonInsurancePayerPayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: NonInsurancePayerPaymentCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[NonInsurancePayerPayment]:
        """
        Parameters
        ----------
        request : NonInsurancePayerPaymentCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[NonInsurancePayerPayment]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/non-insurance-payer-payments/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPayment,
                parse_obj_as(
                    type_=NonInsurancePayerPayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        non_insurance_payer_payment_id: NonInsurancePayerPaymentId,
        *,
        payment_timestamp: typing.Optional[dt.datetime] = OMIT,
        payment_note: typing.Optional[NoteUpdate] = OMIT,
        invoice_id: typing.Optional[InvoiceUpdate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[NonInsurancePayerPayment]:
        """
        Parameters
        ----------
        non_insurance_payer_payment_id : NonInsurancePayerPaymentId

        payment_timestamp : typing.Optional[dt.datetime]

        payment_note : typing.Optional[NoteUpdate]

        invoice_id : typing.Optional[InvoiceUpdate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[NonInsurancePayerPayment]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payer-payments/v1/{jsonable_encoder(non_insurance_payer_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "payment_timestamp": payment_timestamp,
                "payment_note": payment_note,
                "invoice_id": invoice_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPayment,
                parse_obj_as(
                    type_=NonInsurancePayerPayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        non_insurance_payer_payment_id: NonInsurancePayerPaymentId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Deletes the non-insurance payer payment record matching the provided `non_insurance_payer_payment_id`.

        Parameters
        ----------
        non_insurance_payer_payment_id : NonInsurancePayerPaymentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payer-payments/v1/{jsonable_encoder(non_insurance_payer_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        non_insurance_payer_id: typing.Optional[NonInsurancePayerId] = None,
        check_number: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        invoice_id: typing.Optional[InvoiceId] = None,
        sort: typing.Optional[NonInsurancePayerPaymentSortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[NonInsurancePayerPaymentsPage]:
        """
        Returns all non-insurance payer payments

        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100. The value must be greater than 0 and less than 1000.

        non_insurance_payer_id : typing.Optional[NonInsurancePayerId]

        check_number : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        invoice_id : typing.Optional[InvoiceId]

        sort : typing.Optional[NonInsurancePayerPaymentSortField]
            Defaults to refund_timestamp

        sort_direction : typing.Optional[SortDirection]
            Sort direction. Defaults to descending order if not provided.

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[NonInsurancePayerPaymentsPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/non-insurance-payer-payments/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "non_insurance_payer_id": non_insurance_payer_id,
                "check_number": check_number,
                "invoice_id": invoice_id,
                "sort": sort,
                "sort_direction": sort_direction,
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPaymentsPage,
                parse_obj_as(
                    type_=NonInsurancePayerPaymentsPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        non_insurance_payer_payment_id: NonInsurancePayerPaymentId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[NonInsurancePayerPayment]:
        """
        Retrieves a previously created non-insurance payer payment by its `non_insurance_payer_payment_id`.

        Parameters
        ----------
        non_insurance_payer_payment_id : NonInsurancePayerPaymentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[NonInsurancePayerPayment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payer-payments/v1/{jsonable_encoder(non_insurance_payer_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPayment,
                parse_obj_as(
                    type_=NonInsurancePayerPayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: NonInsurancePayerPaymentCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[NonInsurancePayerPayment]:
        """
        Parameters
        ----------
        request : NonInsurancePayerPaymentCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[NonInsurancePayerPayment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/non-insurance-payer-payments/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPayment,
                parse_obj_as(
                    type_=NonInsurancePayerPayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        non_insurance_payer_payment_id: NonInsurancePayerPaymentId,
        *,
        payment_timestamp: typing.Optional[dt.datetime] = OMIT,
        payment_note: typing.Optional[NoteUpdate] = OMIT,
        invoice_id: typing.Optional[InvoiceUpdate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[NonInsurancePayerPayment]:
        """
        Parameters
        ----------
        non_insurance_payer_payment_id : NonInsurancePayerPaymentId

        payment_timestamp : typing.Optional[dt.datetime]

        payment_note : typing.Optional[NoteUpdate]

        invoice_id : typing.Optional[InvoiceUpdate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[NonInsurancePayerPayment]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payer-payments/v1/{jsonable_encoder(non_insurance_payer_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "payment_timestamp": payment_timestamp,
                "payment_note": payment_note,
                "invoice_id": invoice_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                NonInsurancePayerPayment,
                parse_obj_as(
                    type_=NonInsurancePayerPayment,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        non_insurance_payer_payment_id: NonInsurancePayerPaymentId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Deletes the non-insurance payer payment record matching the provided `non_insurance_payer_payment_id`.

        Parameters
        ----------
        non_insurance_payer_payment_id : NonInsurancePayerPaymentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/non-insurance-payer-payments/v1/{jsonable_encoder(non_insurance_payer_payment_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
